// database column container                                     -*- C++ -*-
// $Id: column.h,v 1.1 2004/01/17 23:30:44 roger Exp $
//
// Copyright (C) 2003  Roger Leigh <rleigh@debian.org>
//
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef PQXX_OBJECT_COLUMN_H
#define PQXX_OBJECT_COLUMN_H

#include <sigc++/signal.h>

namespace pqxxobject
{
  template<typename T>
  class column
  {
  public:
    typedef T value_type;

    /// The constructor
    column():
      m_value()
    {}

    /// The constructor
    column(const value_type& value):
      m_value(value)
    {}

    /// The copy constructor
    column(const column<value_type>& rhs):
      m_value(rhs.m_value),
      m_signal_changed()
    {}

    virtual ~column()
    {}

    column<value_type>& operator = (const column<value_type>& rhs)
    {
      set_value(rhs.m_value);
      return *this;
    }

    column<value_type>& operator = (const value_type& rhs)
    {
      set_value(rhs);
      return *this;
    }

    operator const value_type& () const
    {
      return m_value;
    }

    value_type& get_value()
    {
      return m_value;
    }

    void set_value(const value_type& value)
    {
      m_value = value;
      m_signal_changed.emit();
    }

    SigC::Signal0<void>& signal_changed()
    {
      return m_signal_changed;
    }

  private:
    value_type m_value;
    SigC::Signal0<void> m_signal_changed;

  }; // class column

}; // namespace pqxxobject

#endif // PQXX_OBJECT_COLUMN_H
