// tutorial places class                                         -*- C++ -*-
// $Id: places.cc,v 1.5 2004/01/17 23:30:46 roger Exp $
//
// Copyright (C) 2003  Roger Leigh <rleigh@debian.org>
//
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include <sstream>

#include <pqxx-object/exceptions.h>

#include "places.h"

using std::ostringstream;
using std::string;
using std::vector;

using namespace pqxxobject;

Place::Place():
  m_id(0),
  m_name(""),
  m_gridref("")
{
}

Place::Place(const std::string& name,
	     const std::string& gridref):
  m_id(0),
  m_name(name),
  m_gridref(gridref)
{
}

Place::~Place()
{
}

int
Place::get_id() const
{
  return m_id;
}

const std::string&
Place::get_name() const
{
  return m_name;
}

void
Place::set_name(const std::string& name)
{
  m_name = name;
}

const std::string&
Place::get_gridref() const
{
  return m_gridref;
}

void
Place::set_gridref(const std::string& gridref)
{
  m_gridref = gridref;
}

Place::row_ptr
Place::create(pqxx::result::const_iterator row,
	      pqxxobject::transaction& tran)
{
  row_ptr p(new Place);
  row->at("id").to(p->m_id.get_value());
  row->at("name").to(p->m_name.get_value());
  row->at("gridref").to(p->m_gridref.get_value());
  return p;
}

void
Place::insert_impl(pqxxobject::transaction& tran)
{
  ostringstream query;

  query << "INSERT INTO places (name, gridref) "
	<< "VALUES ('" << get_name()
	<< "', '" << get_gridref() << "')";

  tran.perform(query.str(), 1, 1);
}

void
Place::update_impl(pqxxobject::transaction& tran)
{
  ostringstream query;

  query << "UPDATE places "
	<< "SET name = '" << get_name()
	<< "', gridref = '" << get_gridref() << "' "
	<< "WHERE (id = " << get_id() << ")";

  tran.perform(query.str(), 1, 1);
}

void
Place::erase_impl(pqxxobject::transaction& tran)
{
  ostringstream query;

  query << "DELETE FROM places "
	<< "WHERE (id = " << get_id() << ")";

  tran.perform(query.str(), 1, 1);
}

void
Place::refresh_impl(pqxxobject::transaction& tran)
{
}


PlaceTable::PlaceTable(pqxxobject::transaction& tran):
  table_base(tran)
{
}

PlaceTable::~PlaceTable()
{
}

PlaceTable::row_list_ptr
PlaceTable::get_list(sort_order order)
{
  ostringstream query;
  query << "SELECT id, name, gridref FROM places "
	<< "ORDER BY ";

  if (order == ORDER_ID)
    query << "id";
  else if (order == ORDER_NAME)
    query << "name";
  else if (order == ORDER_GRIDREF)
    query << "gridref";
  else // fallback
    query << "name";

  return find_many(query.str());
}

PlaceTable::row_ptr
PlaceTable::find(int place_id)
{
  ostringstream query;
  query << "SELECT id, name, gridref FROM places "
	<< "WHERE (id = " << place_id << ")";

  return find_one(query.str());
}


PlaceTable::row_ptr
PlaceTable::find_name(const std::string& name)
{
  ostringstream query;
  query << "SELECT id, name, gridref FROM places "
	<< "WHERE (name = '" << name << "')";

  return find_one(query.str());
}

PlaceTable::row_list_ptr
PlaceTable::find_gridref(const std::string& gridref)
{
  ostringstream query;
  query << "SELECT id, name, gridref FROM places "
	<< "WHERE (gridref = '" << gridref << "')";

  return find_many(query.str());
}
